/******************************************************************************
 * Programme d'exemple permettant de :
 * - Lire la fréquence du processeur
 * - Accéder à l'instruction RDTSC
 * - Chronométrer très précisément une durée.
 *
 * Par Haypo (victor.stinner@haypocalc.com) - http://www.haypocalc.com/
 *****************************************************************************/

#include "Chronometre.h"

using namespace std;

// Instruction RDTSC du processeur Pentium
double RDTSC(void)
{
    //-----------------Pour LINUX/UNIX-------------
    //unsigned long long x; //Compatible avec gcc enlevant les warning
    unsigned long long x;
    __asm__ volatile  (".byte 0x0f, 0x31" : "=A"(x));
    return (double)x;
}
// Lit la fréquence du processeur
// Renvoie la fréquence en Hz dans 'frequence' si le code de retour est
// différent de 1. Renvoie 0 en cas d'erreur.
int FrequenceCpu (double* frequence)
{
    const char* prefixe_cpu_mhz = "cpu MHz";
    FILE* F;
    char ligne[300+1];
    char *pos;
    int ok=0;

    // Ouvre le fichier
    F = fopen(NOMFICH_CPUINFO, "r");
    if (!F) return 0;

    // Lit une ligne apres l'autre
    while (!feof(F))
    {
        // Lit une ligne de texte
        fgets (ligne, sizeof(ligne), F);

        // C'est la ligne contenant la frequence?
        if (!strncmp(ligne, prefixe_cpu_mhz, strlen(prefixe_cpu_mhz)))
        {
        // Oui, alors lit la frequence
        pos = strrchr (ligne, ':') +2;
        if (!pos) break;
        if (pos[strlen(pos)-1] == '\n') pos[strlen(pos)-1] = '\0';
        strcpy (ligne, pos);
        strcat (ligne,"e6");
        *frequence = atof (ligne);
        ok = 1;
        break;
        }
    }
    fclose (F);
    return 1;
}
//---------------------------------------------------------------------------
/**
    *Retour au programme de calcul de la FFT.
    *Pour la mesure du temps de calcul, on effectuera les opérations suivantes :
        1 - On stocke le nombre de cycles d'horloge processeur
depuis la mise en fonctionnement de la machine = Ci
        2 - On effectue l'algorithme
        3 - On récupère le nombre de cycles = Cf
        4 - Tps écoulé = (Cf-Ci)/Frequence du processeur (donne le temps en secondes)

    *AVANTAGE : Précision à 1/Frequence près !
**/
double Calculs(Complexe Coef[], float ListeR[], int N, int choix)
{
    if(choix==1) //Si l'on a demandé un calcul direct :
    {
        double t = RDTSC();
        FFT(Coef,ListeR,N);
        return (RDTSC() - t);
    }
    else if(choix == 2)
    {

        //La FFTR nécéssite une Liste Complexe
        //On transforme donc la liste R en liste C
        Complexe *ListeC = new Complexe[N];
        for (int i=0;i<N;i++)
        {
            ListeC[i].setRe(ListeR[i]);
        }

        double t = RDTSC();
        FFTR(Coef,ListeC,N);
        t = RDTSC() - t;
        delete[] ListeC;
        return t;
    }
    else if(choix == 3)
    {
        //Si l'on teste la méthode par retournement de bits.
        double t = RDTSC();
        FFTB(Coef,ListeR,N);
        return (RDTSC() - t);
    }
    return 0;
}
